@@ -1,5 +1,5 @@
 from parcomb.char import *
-from .common import assert_success, assert_failure
+from tests.common import assert_success, assert_failure
 
 
 def test_any():
